//封装自 https://ant.design/components/upload-cn 。仍可根据业务扩展
import React, { useState } from 'react';
import { Upload } from 'antd';
import ImgCrop from 'antd-img-crop';
import type { RcFile, UploadFile, UploadProps } from 'antd/es/upload/interface';
import { MdOutlineAddLink } from "react-icons/md";
import { Box } from '@chakra-ui/react';

interface Porps {
    action: string; //上传地址 必选
    maxlength: number;//最大上传数
    getOnChange?: (file: Array<object>) => void; //向父级激活onchange
    postData: object; //上传文件时携带的参数
    headers: any; //上传头部
    crop: boolean;//是否需要剪切
    cropmodalWidth: number;//剪切的倍率，默认100为正方形，500为 1 ：5;
}
const Uploader: React.FC<Porps> = ({ action, maxlength, crop, cropmodalWidth, getOnChange, postData, headers }) => {
    const [fileList, setFileList] = useState<UploadFile[]>([]);

    // {
    //     uid: '-1',
    //     name: 'image.png',
    //     status: 'done',
    //     url: 'https://zos.alipayobjects.com/rmsportal/jkjgkEfvpUPVyRjUImniVslZfWPnJuuZ.png',
    // },

    const onChange: UploadProps['onChange'] = ({ fileList: newFileList }) => {
        setFileList(newFileList);
        getOnChange && getOnChange(newFileList);
    };

    const onPreview = async (file: UploadFile) => {
        let src = file.url as string;
        if (!src) {
            src = await new Promise((resolve) => {
                const reader = new FileReader();
                reader.readAsDataURL(file.originFileObj as RcFile);
                reader.onload = () => resolve(reader.result as string);
            });
        }
        const image = new Image();
        image.src = src;
        const imgWindow = window.open(src);
        imgWindow?.document.write(image.outerHTML);
    };

    return (
        <Box>{
            crop ?
                <ImgCrop rotate modalWidth={cropmodalWidth}>
                    < Upload
                        data={{postData}}
                        headers={headers}
                        action={action}
                        listType="picture-card"
                        fileList={fileList}
                        onChange={onChange}
                        onPreview={onPreview}
                    >
                        {fileList.length < maxlength || 1 && <MdOutlineAddLink size={20} />}
                    </Upload >
                </ImgCrop >
                :
                <Upload
                    data={{postData}}
                    headers={headers}
                    action={action}
                    listType="picture-card"
                    fileList={fileList}
                    onChange={onChange}
                    onPreview={onPreview}
                >
                    {fileList.length < maxlength || 1 && <MdOutlineAddLink size={20} />}
                </Upload>
        }
        </Box>
    );
};

export default Uploader;
