import {
  Table,
  Thead,
  Tbody,
  Tr,
  Th,
  Td,
  TableContainer,
  InputGroup,
  Input,
  InputRightElement,
  Flex,
  FormControl,
  FormLabel,
  FormHelperText,
  Select,
  Card,
  Menu,
  MenuButton,
  MenuList,
  MenuItem
} from '@chakra-ui/react'
import { Box, Button, useToast } from "@chakra-ui/react"
import { Pagination } from 'antd';
import { useState } from "react";
import { FaPlus } from "react-icons/fa";
import AlertDialogExample from "../../components/alert";
import dep from "../../utils/deepClone";
function ApiListConfig() {
  const [menu, setMenu] = useState([]);
  const toast = useToast();
  const [alertRef, setAlertRef] = useState(Object);
  const [showAdd, setShowAdd] = useState(false);
  const [showAddGroup, setShowAddGroup] = useState(false);
  function alertCallback(callback: any) {
    switch (callback.key) {
      case "setApiState":
        toast({
          title: '操作成功',
          status: "success",
          isClosable: true,
        })
        break;
      default:
        break;
    }

  }
  function save() {
    toast({
      title: '保存成功',
      status: "success",
      isClosable: true,
    })
  }
  function setApiState(status: boolean) {
    alertRef.open({
      title: status ? "启用" + "接口 ?" : "停用" + "该接口 ?",
      desc: status ? "恢复该接口的连通性" : "该接口会立即被停用",
      colorScheme: status ? "blue" : "red",
      callback: { key: "setApiState", options: status }
    });
  }
  return (
    <Box>
      <Box className='header-box-extend'>
        <Button colorScheme={showAdd ? 'green' : 'facebook'} variant='ghost' leftIcon={<FaPlus />} onClick={() => setShowAdd(!showAdd)} >
          配置新接口
        </Button>
        <Button colorScheme={showAddGroup ? 'green' : 'facebook'} variant='ghost' leftIcon={<FaPlus />} onClick={() => setShowAddGroup(!showAddGroup)} >
          添加新分类
        </Button>
        <InputGroup size='md' w="16rem">
          <Input
            pr='4.5rem'
            placeholder='搜索接口名称或地址'
          />
          <InputRightElement width='4.5rem'>
            <Button h='1.75rem' size='sm'>
              搜索
            </Button>
          </InputRightElement>
        </InputGroup>
        <Pagination simple defaultCurrent={2} total={50} />
      </Box>
      <AlertDialogExample setting={(res: Function) => setAlertRef(res)} confirm={(callback: any) => alertCallback(callback)} />
      <Flex>
        <Card w="200px" px={2} py={4} h="calc(100vh - 100px)" variant="outline">
          {showAddGroup &&
            <InputGroup size='md' w="100%" mb={8}>
              <Input
                pr='4.5rem'
                placeholder='添加新分组'
              />
              <InputRightElement width='4.5rem'>
                <Button h='1.75rem' size='sm'>
                  Add
                </Button>
              </InputRightElement>
            </InputGroup>
          }
          <Button colorScheme='facebook' w='100%' mb={4} variant='solid'>用户类</Button>
          <Button colorScheme='facebook' w='100%' mb={4} variant='ghost'>商品类</Button>
          <Button colorScheme='facebook' w='100%' mb={4} variant='ghost'>未分组</Button>

        </Card>
        <TableContainer flex="1">
          <Table>
            <Thead>
              <Tr>
                <Th>接口名称</Th>
                <Th>接口目标地址</Th>
                <Th textAlign={"center"}>操作</Th>
              </Tr>
            </Thead>
            <Tbody>
              <Tr>
                <Td>添加商品</Td>
                <Td>/apis/addProduct</Td>
                <Td display={"flex"} gap={4} justifyContent={"center"}>
                  <Button size={"xs"} colorScheme={"red"} onClick={() => setApiState(false)}>停用</Button>
                  <Menu>
                    <MenuButton size={"xs"} as={Button} colorScheme={"facebook"}>
                      移动分组
                    </MenuButton>
                    <MenuList>
                      <MenuItem>商品类</MenuItem>
                      <MenuItem>用户类</MenuItem>
                    </MenuList>
                  </Menu>
                </Td>
              </Tr>
              <Tr>
                <Td>添加商品</Td>
                <Td>/apis/addProduct</Td>
                <Td display={"flex"} gap={4} justifyContent={"center"}>
                  <Button size={"xs"} colorScheme={"red"} onClick={() => setApiState(false)}>停用</Button>
                  <Menu>
                    <MenuButton size={"xs"} as={Button} colorScheme={"facebook"}>
                      移动分组
                    </MenuButton>
                    <MenuList>
                      <MenuItem>商品类</MenuItem>
                      <MenuItem>用户类</MenuItem>
                    </MenuList>
                  </Menu>
                </Td>
              </Tr>
              <Tr>
                <Td>添加商品</Td>
                <Td>/apis/addProduct</Td>
                <Td display={"flex"} gap={4} justifyContent={"center"}>
                  <Button size={"xs"} colorScheme={"gray"} onClick={() => setApiState(true)}>已停用</Button>
                  <Menu>
                    <MenuButton size={"xs"} as={Button} colorScheme={"facebook"}>
                      移动分组
                    </MenuButton>
                    <MenuList>
                      <MenuItem>商品类</MenuItem>
                      <MenuItem>用户类</MenuItem>
                    </MenuList>
                  </Menu>
                </Td>
              </Tr>
            </Tbody>
          </Table>

        </TableContainer>
        <Box transition={"width 400ms"} overflowX={"hidden"} height="calc(100vh - 80px)" w={showAdd ? "300px" : "0"} flexShrink={0} >
          <Box w={"300px"} p={4}>
            <FormControl>
              <FormLabel>分组管理</FormLabel>
              <Select placeholder='无分组'>
                <option value='1'>商品类</option>
                <option value='2'>用户类</option>
              </Select>
              <FormLabel mt={4}>接口名称</FormLabel>
              <Input type='text' placeholder="新接口的名称" />
              <FormLabel mt={4}>接口目标地址</FormLabel>
              <Input type='text' placeholder="新接口的目标地址" />
              <Button mt={4} colorScheme='facebook' w="100%">添加</Button>
            </FormControl>

          </Box>
        </Box>
      </Flex>
    </Box>
  )
}

export default ApiListConfig