import { Box, Text, Button, FormControl, FormLabel, Switch, Card, CardBody, InputGroup, Input, InputRightElement, useToast } from "@chakra-ui/react"
import {
  Menu,
  MenuButton,
  MenuList,
  MenuItem
} from '@chakra-ui/react'
import { useState } from "react";
import { FaPlus, FaAngleDown } from "react-icons/fa";
import { FiChevronRight } from "react-icons/fi";

import store from "../../store";
import AlertDialogExample from "../../components/alert";
import dep from "../../utils/deepClone";

function SettingMenu() {
  const getARoute = store.auth(((state: { isAllRoute: Object }) => state.isAllRoute));
  const [menu, setMenu] = useState(getARoute);
  const [showPrv, setShowPrv] = useState(false);
  const [alertRef, setAlertRef] = useState(Object);
  const toast = useToast();
  function save() {
    alertRef.open({
      title: "已完成设置吗?",
      desc: "保存后 其他用户 将在下一次登录时使用新目录结构",
      colorScheme: "green",
      callback: "save"
    })
  }
  function saveResult() {
    return toast({
      title: '设置成功',
      status: "success",
      isClosable: true,
    })
  }
  function delMenuItem(item: any, index: number) {
    alertRef.open({
      title: "要移除 " + item.menu_name + " 吗?",
      desc: "移除后所有子项目将被暂存在 “其他” 中",
      callback: index
    })
  }
  function delMenuItemCon(index: number) {

    let newMenu = dep.deepClone(menu);
    newMenu.order_list = newMenu.order_list.concat(menu.menu_list[index].path_list);
    newMenu.menu_list.splice(index, 1);
    setMenu(newMenu);
  }
  function setSeq(f: string, index: number) {

    let newMenu = dep.deepClone(menu);
    switch (f) {
      case 'up':
        [newMenu.menu_list[index - 1], newMenu.menu_list[index]] = [newMenu.menu_list[index], newMenu.menu_list[index - 1]]
        setMenu(newMenu);
        break;
      case 'down':
        [newMenu.menu_list[index + 1], newMenu.menu_list[index]] = [newMenu.menu_list[index], newMenu.menu_list[index + 1]]
        setMenu(newMenu);
        break;
    }
  }
  function setItemSeq(f: string, mIndex: number, index: number) {

    let newMenu = dep.deepClone(menu);
    switch (f) {
      case 'last':
        [newMenu.menu_list[mIndex].path_list[index - 1], newMenu.menu_list[mIndex].path_list[index]] = [newMenu.menu_list[mIndex].path_list[index], newMenu.menu_list[mIndex].path_list[index - 1]]
        setMenu(newMenu);
        break;
      case 'next':
        [newMenu.menu_list[mIndex].path_list[index + 1], newMenu.menu_list[mIndex].path_list[index]] = [newMenu.menu_list[mIndex].path_list[index], newMenu.menu_list[mIndex].path_list[index + 1]]
        setMenu(newMenu);
        break;
    }
  }
  function setItemIndex(f: string, lIndex: number, pIndex: number, mIndex: number) {

    let newMenu = dep.deepClone(menu);
    switch (f) {
      case 'menuList':
        newMenu.menu_list[mIndex].path_list.push(newMenu.menu_list[lIndex].path_list[pIndex]);
        newMenu.menu_list[lIndex].path_list.splice(pIndex, 1)
        setMenu(newMenu);
        break;
      case 'orderList':
        newMenu.menu_list[mIndex].path_list.push(newMenu.order_list[lIndex]);
        newMenu.order_list.splice(lIndex, 1);
        setMenu(newMenu);
        break;
    }
  }
  return (
    <Box display={"flex"}>
      <Box className='header-box-extend'>
        <Text>开启预览？</Text>
        <Switch id='prv-alerts' onChange={() => setShowPrv(!showPrv)} />
        <Button colorScheme='facebook' variant='ghost' leftIcon={<FaPlus />} onClick={() => {
          let newMenu = dep.deepClone(menu);
          newMenu.menu_list.push({ "menu_name": "新目录", "path_list": [] });
          setMenu(newMenu);
        }} >
          添加新目录
        </Button>
        <Button colorScheme='green' onClick={save}>保存</Button>
      </Box>
      <AlertDialogExample setting={(res: Function) => setAlertRef(res)} confirm={(callback: any) => callback == "save" ? saveResult() : delMenuItemCon(callback)} />
      <Box display={"flex"} justifyContent={"center"} transition={"width 400ms"} overflowX={"hidden"} height="calc(100vh - 80px)" w={showPrv ? "200px" : "0"} flexShrink={0} >
        <Box w={"200px"}>
          {
            menu.menu_list.map((item: any, index: any) => (
              <Box key={index}>
                {item.path_list.length > 0 && <Text fontSize={12} color={"gray.400"} px={4} w={"200px"}>{item.menu_name}</Text>}
                {
                  item.path_list.map((citem: any, cindex: any) => (
                    <Button key={cindex} w="100%" mb={"2px"} colorScheme='facebook' variant='ghost'>
                      {citem.name}
                    </Button>
                  ))
                }
              </Box>
            ))
          }
          {menu.order_list.length > 0 && <Text fontSize={12} color={"gray.400"} px={4} w={"200px"}>其他</Text>}
          {
            menu.order_list.map((citem: any, cindex: any) => (
              <Button key={cindex} w="100%" mb={"2px"} colorScheme='facebook' variant='ghost'>
                {citem.name}
              </Button>
            ))
          }
        </Box>
      </Box>
      <Box flex={1}>
        <Card>
          <CardBody>
            {
              menu.menu_list.map((item: any, index: any) => (
                <Box key={index}>
                  <InputGroup size='md'>
                    <Input
                      value={item.menu_name}
                      onChange={(event) => {

                        let newMenu = dep.deepClone(menu);
                        newMenu.menu_list[index].menu_name = event.target.value
                        setMenu(newMenu);
                      }}
                      pr='4.5rem'
                      placeholder='Enter menu name'
                      variant='flushed'
                    />
                    <InputRightElement width='auto'>
                      {index != 0 && <Button h='1.75rem' size='sm' mr={4} onClick={() => setSeq("up", index)}>上移</Button>}
                      {index != menu.menu_list.length - 1 && <Button h='1.75rem' size='sm' mr={4} onClick={() => setSeq("down", index)}>下移</Button>}
                      <Button h='1.75rem' size='sm' onClick={() => delMenuItem(item, index)}>移除</Button>
                    </InputRightElement>
                  </InputGroup>

                  <Box display={"flex"} gap={4} mb={4} mt={1} minH={"60px"}>
                    {
                      item.path_list.map((citem: any, cindex: any) => (
                        <Menu key={cindex}>
                          <MenuButton as={Button} m={"2px"} rightIcon={<FaAngleDown />} colorScheme='facebook'>
                            {citem.name}
                          </MenuButton>
                          <MenuList>
                            {cindex != 0 && <MenuItem onClick={() => setItemSeq("last", index, cindex)}>前移</MenuItem>}
                            {cindex != item.path_list.length - 1 && <MenuItem onClick={() => setItemSeq("next", index, cindex)}>后移</MenuItem>}
                            <Text fontSize={12} color={"gray.400"} px={2}>移动到目录</Text>
                            {
                              menu.menu_list.map((mitem: any, mindex: any) => {
                                return index != mindex && <MenuItem key={mindex} onClick={() => setItemIndex("menuList", index, cindex, mindex)} display="flex" alignContent={"center"} gap={4}><FiChevronRight />{mitem.menu_name}</MenuItem>
                              })
                            }
                          </MenuList>
                        </Menu>
                      ))
                    }
                  </Box>
                </Box>
              ))
            }
            <Text>其他</Text>
            <Box display={"flex"} gap={4} mb={4} mt={1}>
              {
                menu.order_list.map((citem: any, cindex: any) => (
                  <Menu key={cindex}>
                    <MenuButton as={Button} m={"2px"} rightIcon={<FaAngleDown />} colorScheme='facebook'>
                      {citem.name}
                    </MenuButton>
                    <MenuList>
                      <Text fontSize={12} color={"gray.400"} px={2}>移动到目录</Text>
                      {
                        menu.menu_list.map((mitem: any, mindex: any) => {
                          return <MenuItem key={mindex} onClick={() => setItemIndex("orderList", cindex, 0, mindex)} display="flex" alignContent={"center"} gap={4}><FiChevronRight />{mitem.menu_name}</MenuItem>
                        })
                      }
                    </MenuList>
                  </Menu>
                ))
              }
            </Box>
          </CardBody>
        </Card>
      </Box>
    </Box>
  )
}


export default SettingMenu