import {
  Box,
  Table,
  Thead,
  Tbody,
  Tr,
  Th,
  Td,
  TableContainer,
  Button,
  useDisclosure,
  Input,
  Tooltip,
  useConst,
  useToast,
  Flex,
  FormControl,
  Select,
  FormLabel,
} from '@chakra-ui/react'
import {
  Text,
  Drawer,
  DrawerBody,
  DrawerFooter,
  DrawerHeader,
  DrawerOverlay,
  DrawerContent,
  DrawerCloseButton
} from '@chakra-ui/react'
import {
  Menu,
  MenuButton,
  MenuList,
  MenuItem,
} from '@chakra-ui/react'
import { useClipboard } from '@chakra-ui/react'
import { useState } from 'react';
import { FaAngleDown } from 'react-icons/fa';
import { handleExportCurrentExcel } from '../../utils/creatExcel';
import AlertDialogExample from "../../components/alert"
import TreeCheckBox from '../../components/treeCheckBox';
import BwlLink from '../../components/bwlLink';

function System() {
  const toast = useToast();
  const { isOpen, onOpen, onClose } = useDisclosure();
  const [showUserHistory, setShowUserHistory] = useState(false);
  const [showUserAuth, setshowUserAuth] = useState(false);
  const { onCopy, value, setValue, hasCopied } = useClipboard("");
  const [alertRef, setAlertRef] = useState(Object);
  // 表头
  const historyColumns = useConst(
    [
      { code: 'time', title: '操作时间' },
      { code: 'apiAddress', title: '接口地址' },
      { code: 'apiData', title: '传递参数', align: "center", tooltip: true, maxW: "240" }
    ]
  )
  // 数据
  const historyData = useConst(
    [
      { time: '2023-1-10', apiAddress: '/admin/apis', apiData: '"title":"hello"' },
      { time: '2022-4-10', apiAddress: '/admin/open', apiData: '"title":"hello"' },
      { time: '2021-10-10', apiAddress: '/admin/apis/tip', apiData: '无' },
    ]
  )
  function downLoadExcel() {
    const options = {
      fileName: "excel表格",
      columns: historyColumns,
      data: historyData
    }
    handleExportCurrentExcel(options).then(res => {
      toast({
        title: '导出成功',
        status: "success",
        isClosable: true,
      })
    });
  }
  function alertConfirm(callback: any) {
    const data = callback;
    switch (data.key) {
      case "setUserState":
        // 在此处理回调事件
        toast({
          title: '操作成功' + data.options,
          status: "success",
          isClosable: true,
        })
        break;

      default:
        break;
    }
  }
  function setUserState(status: boolean) {
    alertRef.open({
      title: status ? "启用" + "该账号 ?" : "停用" + "该账号 ?",
      desc: status ? "恢复该账号的使用权限" : "该账号会立即被停用并移除登录状态",
      colorScheme: status ? "blue" : "red",
      callback: { key: "setUserState", options: status }
    });
  }
  return (
    <Box>
      <Box className='header-box-extend'>
        <Button colorScheme='blue' variant='ghost' onClick={onOpen}>
          创建新用户
        </Button>
      </Box>
      <AlertDialogExample setting={(res: Function) => setAlertRef(res)} confirm={alertConfirm} />
      <TableContainer>
        <Table>
          <Thead>
            <Tr>
              <Th>用户</Th>
              <Th>登录时间及地址</Th>
              <Th>权限等级</Th>
              <Th textAlign={"center"}>操作</Th>
            </Tr>
          </Thead>
          <Tbody>
            <Tr>
              <Td>inches</Td>
              <Td color={"green"}>当前在线 21.200.49.2 （南宁市青秀区）</Td>
              <Td>次最高权限</Td>
              <Td display={"flex"} gap={4} justifyContent={"center"}>
                <Button size={"xs"} colorScheme={"red"} onClick={() => setUserState(false)}>停用</Button>
                <Button size={"xs"} colorScheme={"green"} onClick={() => setshowUserAuth(true)}>权限管理</Button>
                <Button size={"xs"} colorScheme={"blue"} onClick={() => setShowUserHistory(true)}>用户操作记录</Button>
              </Td>
            </Tr>
            <Tr>
              <Td>feet</Td>
              <Td>昨天  21.200.49.2 （南宁市青秀区） </Td>
              <Td>普通操作员</Td>
              <Td display={"flex"} gap={4} justifyContent={"center"}>
                <Button size={"xs"} colorScheme={"red"}>停用</Button>
                <Button size={"xs"} colorScheme={"green"}>权限管理</Button>
                <Button size={"xs"} colorScheme={"blue"}>用户操作记录</Button>
              </Td>
            </Tr>
            <Tr>
              <Td>yards</Td>
              <Td>2月14日 21.200.49.2 （南宁市青秀区）</Td>
              <Td>仅可读</Td>
              <Td display={"flex"} gap={4} justifyContent={"center"}>
                <Button size={"xs"} colorScheme={"gray"} onClick={() => setUserState(true)}>已停用</Button>
                <Button size={"xs"} colorScheme={"green"}>权限管理</Button>
                <Button size={"xs"} colorScheme={"blue"}>用户操作记录</Button>
              </Td>
            </Tr>
          </Tbody>
        </Table>
      </TableContainer>
      <Drawer
        isOpen={isOpen}
        placement='right'
        onClose={onClose}
        closeOnOverlayClick={false}
      >
        <DrawerOverlay />
        <DrawerContent>
          <DrawerCloseButton />
          <DrawerHeader>创建新用户</DrawerHeader>

          <DrawerBody>
            <Input placeholder='登录用户名' />
            <Text fontSize={12} color={"gray.400"}>由大写字母开头并不小于6位。</Text>
            <Menu>
              <MenuButton mt={4} w={"100%"} as={Button} rightIcon={<FaAngleDown />}>
                （预设权限）自定义
              </MenuButton>
              <MenuList>
                <MenuItem>自定义</MenuItem>
                <MenuItem>次最高等级</MenuItem>
                <MenuItem>普通操作员</MenuItem>
                <MenuItem>仅可读</MenuItem>
              </MenuList>
            </Menu>
            <Text fontSize={12} color={"gray.400"}>默认为自定义等级，用户无任何操作表权限，可在创建完成后单独对用户设置权限。</Text>

            <Box mt={4}>
              <Text>您的账号创建成功 用户名 adminxxx 临时密码 xkdswe3@1{value}</Text>
              <Button mt={2} onClick={onCopy}>{hasCopied ? "已复制!" : "复制"}</Button>
            </Box>
          </DrawerBody>

          <DrawerFooter>
            <Button variant='outline' mr={3} onClick={onClose}>
              取消
            </Button>
            <Button colorScheme='blue'>创建</Button>
          </DrawerFooter>
        </DrawerContent>
      </Drawer>
      <Drawer
        size={"xl"}
        isOpen={showUserHistory}
        placement='right'
        onClose={() => setShowUserHistory(false)}
      >
        <DrawerOverlay />
        <DrawerContent>
          <DrawerCloseButton />
          <DrawerHeader>用户 Emma 操作记录</DrawerHeader>
          <DrawerBody>
            <TableContainer overflow={"hidden"}>
              <Table size={"md"}>
                <Thead>
                  <Tr>
                    {
                      historyColumns.map((item: any, index: any) => (
                        <Th key={index} textAlign={item.align}>{item.title}</Th>
                      ))
                    }
                  </Tr>
                </Thead>
                <Tbody>
                  {
                    historyData.map((item: any, index: any) => (
                      <Tr key={index}>
                        {
                          historyColumns.map((tdItem: any, tdIndex: any) => (
                            <Td key={tdIndex} textAlign={tdItem.align}>
                              {
                                tdItem.tooltip ?
                                  <Tooltip label={item[tdItem.code]}>
                                    <Text maxW={(tdItem.maxW || 400) + "px"} textAlign={tdItem.align} className='table-td-text'>{item[tdItem.code]}</Text>
                                  </Tooltip>
                                  :
                                  <Text maxW={(tdItem.maxW || 400) + "px"} textAlign={tdItem.align} className='table-td-text'>{item[tdItem.code]}</Text>
                              }
                            </Td>
                          ))
                        }
                      </Tr>
                    ))
                  }
                </Tbody>
              </Table>
            </TableContainer>
          </DrawerBody>
          <DrawerFooter>
            <Button color={"green.500"} onClick={downLoadExcel}>
              导出Excel表格
            </Button>
          </DrawerFooter>
        </DrawerContent>
      </Drawer>
      <Drawer
        size={"xl"}
        isOpen={showUserAuth}
        placement='right'
        onClose={() => setshowUserAuth(false)}
      >
        <DrawerOverlay />
        <DrawerContent>
          <DrawerCloseButton />
          <DrawerHeader>用户 Emma 权限管理</DrawerHeader>
          <DrawerBody>
            <FormControl>
              <Flex>
                <FormLabel flexShrink="0" w={100}><Text as={"b"}>使用预设权限</Text></FormLabel>
                <Box flex={1}>
                  <Select placeholder='自定义'>
                    <option>次最高等级</option>
                    <option>普通操作员</option>
                    <option>仅可读</option>
                  </Select>
                </Box>
              </Flex>
              <BwlLink useConfirm desc="配置预设权限" link={"/"} />
              <Flex mt={4}>
                <FormLabel flexShrink="0" w={100}><Text as={"b"}>可用路由表</Text></FormLabel>
                <Box flex={1} >
                  <TreeCheckBox array={[]} />
                </Box>
              </Flex>
              <BwlLink  useConfirm desc="路由表在目录管理中配置" link={"/"} />
              <Flex mt={4}>
                <FormLabel flexShrink="0" w={100}><Text as={"b"}>可用接口表</Text></FormLabel>
                <Box flex={1} >
                  <TreeCheckBox array={[]} />
                </Box>
              </Flex>
              <BwlLink useConfirm desc="接口表在接口管理中配置" link={"/"} />
            </FormControl>

          </DrawerBody>
          <DrawerFooter>
            <Button color={"green"} onClick={downLoadExcel}>
              保存该配置
            </Button>
          </DrawerFooter>
        </DrawerContent>
      </Drawer>
    </Box>
  )
}

export default System